@@ -46,7 +46,7 @@ module Sequel
     # :ignore_errors :: Ignore any DatabaseErrors that are raised
     #
     # See <tt>alter_table</tt>.
-    def add_index(table, columns, options={})
+    def add_index(table, columns, options=OPTS)
       e = options[:ignore_errors]
       begin
         alter_table(table){add_index(columns, options)}
@@ -71,7 +71,7 @@ module Sequel
     # definitions using <tt>create_table</tt>, and +add_index+ accepts all the options
     # available for index definition.
     #
-    # See <tt>Schema::AlterTableGenerator</tt> and the {"Migrations and Schema Modification" guide}[link:files/doc/migration_rdoc.html].
+    # See <tt>Schema::AlterTableGenerator</tt> and the {"Migrations and Schema Modification" guide}[rdoc-ref:doc/migration.rdoc].
     def alter_table(name, generator=nil, &block)
       generator ||= alter_table_generator(&block)
       remove_cached_schema(name)
@@ -117,7 +117,7 @@ module Sequel
     # :name :: The name of the table to create
     # :no_index :: Set to true not to create the second index.
     # :no_primary_key :: Set to true to not create the primary key.
-    def create_join_table(hash, options={})
+    def create_join_table(hash, options=OPTS)
       keys = hash.keys.sort_by{|k| k.to_s}
       create_table(join_table_name(hash, options), options) do
         keys.each do |key|
@@ -155,10 +155,13 @@ module Sequel
     # :engine :: The table engine to use for the table.
     #
     # PostgreSQL specific options:
+    # :on_commit :: Either :preserve_rows (default), :drop or :delete_rows.
     # :unlogged :: Create the table as an unlogged table.
+    # :inherits :: Inherit from a different tables.  An array can be
+    #              specified to inherit from multiple tables.
     #
-    # See <tt>Schema::Generator</tt> and the {"Schema Modification" guide}[link:files/doc/schema_modification_rdoc.html].
-    def create_table(name, options={}, &block)
+    # See <tt>Schema::Generator</tt> and the {"Schema Modification" guide}[rdoc-ref:doc/schema_modification.rdoc].
+    def create_table(name, options=OPTS, &block)
       remove_cached_schema(name)
       options = {:generator=>options} if options.is_a?(Schema::CreateTableGenerator)
       if sql = options[:as]
@@ -178,7 +181,7 @@ module Sequel
     #   # SELECT NULL FROM a LIMIT 1 -- check existence
     #   # DROP TABLE a -- drop table if already exists
     #   # CREATE TABLE a (a integer)
-    def create_table!(name, options={}, &block)
+    def create_table!(name, options=OPTS, &block)
       drop_table?(name)
       create_table(name, options, &block)
     end
@@ -188,7 +191,7 @@ module Sequel
     #   DB.create_table?(:a){Integer :a} 
     #   # SELECT NULL FROM a LIMIT 1 -- check existence
     #   # CREATE TABLE a (a integer) -- if it doesn't already exist
-    def create_table?(name, options={}, &block)
+    def create_table?(name, options=OPTS, &block)
       if supports_create_table_if_not_exists?
         create_table(name, options.merge(:if_not_exists=>true), &block)
       elsif !table_exists?(name)
@@ -202,24 +205,48 @@ module Sequel
       create_table_generator_class.new(self, &block)
     end
     
-    # Creates a view, replacing it if it already exists:
+    # Creates a view, replacing a view with the same name if one already exists.
     #
-    #   DB.create_or_replace_view(:cheap_items, "SELECT * FROM items WHERE price < 100")
-    #   DB.create_or_replace_view(:ruby_items, DB[:items].filter(:category => 'ruby'))
-    def create_or_replace_view(name, source)
-      source = source.sql if source.is_a?(Dataset)
-      execute_ddl("CREATE OR REPLACE VIEW #{quote_schema_table(name)} AS #{source}")
-      remove_cached_schema(name)
-      nil
+    #   DB.create_or_replace_view(:some_items, "SELECT * FROM items WHERE price < 100")
+    #   DB.create_or_replace_view(:some_items, DB[:items].filter(:category => 'ruby'))
+    #
+    # For databases where replacing a view is not natively supported, support
+    # is emulated by dropping a view with the same name before creating the view.
+    def create_or_replace_view(name, source, options = OPTS)
+      if supports_create_or_replace_view?
+        options = options.merge(:replace=>true)
+      else
+        drop_view(name) rescue nil
+      end
+
+      create_view(name, source, options)
     end
     
     # Creates a view based on a dataset or an SQL string:
     #
     #   DB.create_view(:cheap_items, "SELECT * FROM items WHERE price < 100")
     #   DB.create_view(:ruby_items, DB[:items].filter(:category => 'ruby'))
-    def create_view(name, source)
-      source = source.sql if source.is_a?(Dataset)
-      execute_ddl("CREATE VIEW #{quote_schema_table(name)} AS #{source}")
+    #
+    # Options:
+    # :columns :: The column names to use for the view.  If not given,
+    #             automatically determined based on the input dataset.
+    #
+    # PostgreSQL/SQLite specific option:
+    # :temp :: Create a temporary view, automatically dropped on disconnect.
+    #
+    # PostgreSQL specific option:
+    # :materialized :: Creates a materialized view, similar to a regular view,
+    #                  but backed by a physical table.
+    # :recursive :: Creates a recursive view.  As columns must be specified for
+    #               recursive views, you can also set them as the value of this
+    #               option.  Since a recursive view requires a union that isn't
+    #               in a subquery, if you are providing a Dataset as the source
+    #               argument, if should probably call the union method with the
+    #               :all=>true and :from_self=>false options.
+    def create_view(name, source, options = OPTS)
+      execute_ddl(create_view_sql(name, source, options))
+      remove_cached_schema(name)
+      nil
     end
     
     # Removes a column from the specified table:
@@ -237,7 +264,7 @@ module Sequel
     #   DB.drop_index :posts, [:author, :title]
     #
     # See <tt>alter_table</tt>.
-    def drop_index(table, columns, options={})
+    def drop_index(table, columns, options=OPTS)
       alter_table(table){drop_index(columns, options)}
     end
 
@@ -246,7 +273,7 @@ module Sequel
     #
     #   drop_join_table(:cat_id=>:cats, :dog_id=>:dogs)
     #   # DROP TABLE cats_dogs
-    def drop_join_table(hash, options={})
+    def drop_join_table(hash, options=OPTS)
       drop_table(join_table_name(hash, options), options)
     end
     
@@ -289,6 +316,13 @@ module Sequel
     #   DB.drop_view(:cheap_items)
     #   DB.drop_view(:cheap_items, :pricey_items)
     #   DB.drop_view(:cheap_items, :pricey_items, :cascade=>true)
+    #
+    # Options:
+    # :cascade :: Also drop objects depending on this view.
+    #
+    # PostgreSQL specific options:
+    # :if_exists :: Do not raise an error if the view does not exist.
+    # :materialized :: Drop a materialized view.
     def drop_view(*names)
       options = names.last.is_a?(Hash) ? names.pop : {}
       names.each do |n|
@@ -373,6 +407,9 @@ module Sequel
       when :add_constraint
         "ADD #{constraint_definition_sql(op)}"
       when :drop_constraint
+        if op[:type] == :foreign_key
+          quoted_name ||= quote_identifier(foreign_key_name(table, op[:columns]))
+        end
         "DROP CONSTRAINT #{quoted_name}#{' CASCADE' if op[:cascade]}"
       else
         raise Error, "Unsupported ALTER TABLE operation: #{op[:op]}"
@@ -406,7 +443,7 @@ module Sequel
               last_combinable = true
             end
           elsif sql = alter_table_sql(table, op)
-            grouped_ops << sql
+            Array(sql).each{|s| grouped_ops << s}
             last_combinable = false
           end
         end
@@ -464,17 +501,32 @@ module Sequel
     
     # Add primary key SQL fragment to column creation SQL.
     def column_definition_primary_key_sql(sql, column)
-      sql << PRIMARY_KEY if column[:primary_key]
+      if column[:primary_key]
+        if name = column[:primary_key_constraint_name]
+          sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << PRIMARY_KEY
+      end
     end
     
     # Add foreign key reference SQL fragment to column creation SQL.
     def column_definition_references_sql(sql, column)
-      sql << column_references_column_constraint_sql(column) if column[:table]
+      if column[:table]
+        if name = column[:foreign_key_constraint_name]
+         sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << column_references_column_constraint_sql(column)
+      end
     end
     
     # Add unique constraint SQL fragment to column creation SQL.
     def column_definition_unique_sql(sql, column)
-      sql << UNIQUE if column[:unique]
+      if column[:unique]
+        if name = column[:unique_constraint_name]
+          sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << UNIQUE
+      end
     end
     
     # SQL for all given columns, used inside a CREATE TABLE block.
@@ -514,15 +566,18 @@ module Sequel
       case constraint[:type]
       when :check
         check = constraint[:check]
-        sql << "CHECK #{filter_expr((check.is_a?(Array) && check.length == 1) ? check.first : check)}"
+        check = check.first if check.is_a?(Array) && check.length == 1
+        check = filter_expr(check)
+        check = "(#{check})" unless check[0..0] == '(' && check[-1..-1] == ')'
+        sql << "CHECK #{check}"
       when :primary_key
         sql << "PRIMARY KEY #{literal(constraint[:columns])}"
       when :foreign_key
-        sql << column_references_table_constraint_sql(constraint)
+        sql << column_references_table_constraint_sql(constraint.merge(:deferrable=>nil))
       when :unique
         sql << "UNIQUE #{literal(constraint[:columns])}"
       else
-        raise Error, "Invalid constriant type #{constraint[:type]}, should be :check, :primary_key, :foreign_key, or :unique"
+        raise Error, "Invalid constraint type #{constraint[:type]}, should be :check, :primary_key, :foreign_key, or :unique"
       end
       constraint_deferrable_sql_append(sql, constraint[:deferrable])
       sql
@@ -565,6 +620,23 @@ module Sequel
 
     # DDL statement for creating a table with the given name, columns, and options
     def create_table_sql(name, generator, options)
+      unless supports_named_column_constraints?
+        # Split column constraints into table constraints if they have a name
+        generator.columns.each do |c|
+          if (constraint_name = c.delete(:foreign_key_constraint_name)) && (table = c.delete(:table))
+            opts = {}
+            opts[:name] = constraint_name
+            [:key, :on_delete, :on_update, :deferrable].each{|k| opts[k] = c[k]}
+            generator.foreign_key([c[:name]], table, opts)
+          end
+          if (constraint_name = c.delete(:unique_constraint_name)) && c.delete(:unique)
+            generator.unique(c[:name], :name=>constraint_name)
+          end
+          if (constraint_name = c.delete(:primary_key_constraint_name)) && c.delete(:primary_key)
+            generator.primary_key([c[:name]], :name=>constraint_name)
+          end
+        end
+      end
       "#{create_table_prefix_sql(name, options)} (#{column_list_sql(generator)})"
     end
 
@@ -581,18 +653,46 @@ module Sequel
       "#{create_table_prefix_sql(name, options)} AS #{sql}"
     end
 
-    # DDL statement for creating a table with the given name, columns, and options
+    # DDL fragment for initial part of CREATE TABLE statement
     def create_table_prefix_sql(name, options)
       "CREATE #{temporary_table_sql if options[:temp]}TABLE#{' IF NOT EXISTS' if options[:if_not_exists]} #{options[:temp] ? quote_identifier(name) : quote_schema_table(name)}"
     end
 
+    # DDL fragment for initial part of CREATE VIEW statement
+    def create_view_prefix_sql(name, options)
+      create_view_sql_append_columns("CREATE #{'OR REPLACE 'if options[:replace]}VIEW #{quote_schema_table(name)}", options[:columns])
+    end
+
+    # DDL statement for creating a view.
+    def create_view_sql(name, source, options)
+      source = source.sql if source.is_a?(Dataset)
+      "#{create_view_prefix_sql(name, options)} AS #{source}"
+    end
+
+    # Append the column list to the SQL, if a column list is given.
+    def create_view_sql_append_columns(sql, columns)
+      if columns
+        sql << ' ('
+        schema_utility_dataset.send(:identifier_list_append, sql, columns)
+        sql << ')'
+      end
+      sql
+    end
+
     # Default index name for the table and columns, may be too long
     # for certain databases.
     def default_index_name(table_name, columns)
       schema, table = schema_and_table(table_name)
-      "#{"#{schema}_" if schema and schema != default_schema}#{table}_#{columns.map{|c| [String, Symbol].any?{|cl| c.is_a?(cl)} ? c : literal(c).gsub(/\W/, '_')}.join(UNDERSCORE)}_index"
+      "#{"#{schema}_" if schema}#{table}_#{columns.map{|c| [String, Symbol].any?{|cl| c.is_a?(cl)} ? c : literal(c).gsub(/\W/, '_')}.join(UNDERSCORE)}_index"
     end
   
+    # Get foreign key name for given table and columns.
+    def foreign_key_name(table_name, columns)
+      keys = foreign_key_list(table_name).select{|key| key[:columns] == columns}
+      raise(Error, "#{keys.empty? ? 'Missing' : 'Ambiguous'} foreign key for #{columns.inspect}") unless keys.size == 1
+      keys.first[:name]
+    end
+    
     # The SQL to drop an index for the table.
     def drop_index_sql(table, op)
       "DROP INDEX #{quote_identifier(op[:name] || default_index_name(table, op[:columns]))}"
@@ -617,13 +717,9 @@ module Sequel
     # and index specifications.
     def index_definition_sql(table_name, index)
       index_name = index[:name] || default_index_name(table_name, index[:columns])
-      if index[:type]
-        raise Error, "Index types are not supported for this database"
-      elsif index[:where]
-        raise Error, "Partial indexes are not supported for this database"
-      else
-        "CREATE #{'UNIQUE ' if index[:unique]}INDEX #{quote_identifier(index_name)} ON #{quote_schema_table(table_name)} #{literal(index[:columns])}"
-      end
+      raise Error, "Index types are not supported for this database" if index[:type]
+      raise Error, "Partial indexes are not supported for this database" if index[:where] && !supports_partial_indexes?
+      "CREATE #{'UNIQUE ' if index[:unique]}INDEX #{quote_identifier(index_name)} ON #{quote_schema_table(table_name)} #{literal(index[:columns])}#{" WHERE #{filter_expr(index[:where])}" if index[:where]}"
     end
   
     # Array of SQL DDL statements, one for each index specification,
@@ -684,22 +780,11 @@ module Sequel
       schema_utility_dataset.quote_schema_table(table)
     end
     
-    # Proxy the quote_identifier method to the dataset, used for quoting tables and columns.
-    def quote_identifier(v)
-      schema_utility_dataset.quote_identifier(v)
-    end
-    
     # SQL DDL statement for renaming a table.
     def rename_table_sql(name, new_name)
       "ALTER TABLE #{quote_schema_table(name)} RENAME TO #{quote_schema_table(new_name)}"
     end
 
-    # Remove the cached schema_utility_dataset, because the identifier
-    # quoting has changed.
-    def reset_schema_utility_dataset
-      @schema_utility_dataset = nil
-    end
-    
     # Split the schema information from the table
     def schema_and_table(table_name)
       schema_utility_dataset.schema_and_table(table_name)
@@ -712,7 +797,7 @@ module Sequel
 
     # The dataset to use for proxying certain schema methods.
     def schema_utility_dataset
-      @schema_utility_dataset ||= dataset
+      @default_dataset
     end
 
     # Split the schema information from the table
@@ -720,12 +805,6 @@ module Sequel
       schema_utility_dataset.split_qualifiers(table_name)
     end
 
-    # Whether the database supports combining multiple alter table
-    # operations into a single query, false by default.
-    def supports_combining_alter_table_ops?
-      false
-    end
-
     # SQL DDL fragment for temporary table
     def temporary_table_sql
       self.class.const_get(:TEMPORARY)
