@@ -1,6 +1,6 @@
-module Sequel
-  tsk_require 'nokogiri'
+require 'nokogiri'
 
+module Sequel
   module Plugins
     # The xml_serializer plugin handles serializing entire Sequel::Model
     # objects to XML, and deserializing XML into a single Sequel::Model
@@ -45,7 +45,7 @@ module Sequel
     # You can use a hash value with <tt>:include</tt> to pass options
     # to associations:
     #
-    #   album.to_json(:include=>{:artist=>{:only=>:name}})
+    #   album.to_xml(:include=>{:artist=>{:only=>:name}})
     #   # Output:
     #   <?xml version="1.0"?>
     #   <album>
@@ -57,27 +57,51 @@ module Sequel
     #     </artist>
     #   </album>
     #
-    # In addition to creating XML, this plugin also enables Sequel::Model
-    # objects to be created by parsing XML:
-    #
-    #   xml = album.to_xml
-    #   album = Album.from_xml(xml)
-    #
-    # In addition, you can update existing model objects directly from XML
-    # using +from_xml+:
-    #
-    #   album.from_xml(xml)
-    #
-    # Additionally, +to_xml+ also exists as a class and dataset method, both
+    # +to_xml+ also exists as a class and dataset method, both
     # of which return all objects in the dataset:
     #
     #   Album.to_xml
     #   Album.filter(:artist_id=>1).to_xml(:include=>:tags)
     #
-    # Such XML can be loaded back into an array of Sequel::Model objects using
-    # +array_from_xml+:
+    # If you have an existing array of model instances you want to convert to
+    # XML, you can call the class to_xml method with the :array option:
     #
-    #   Album.array_from_xml(Album.to_xml) # same as Album.all
+    #   Album.to_xml(:array=>[Album[1], Album[2]])
+    #
+    # In addition to creating XML, this plugin also enables Sequel::Model
+    # classes to create instances directly from XML using the from_xml class
+    # method:
+    #
+    #   xml = album.to_xml
+    #   album = Album.from_xml(xml)
+    #
+    # The array_from_xml class method exists to parse arrays of model instances
+    # from xml:
+    #
+    #   xml = Album.filter(:artist_id=>1).to_xml
+    #   albums = Album.array_from_xml(xml)
+    #
+    # These does not necessarily round trip, since doing so would let users
+    # create model objects with arbitrary values.  By default, from_xml will
+    # call set using values from the tags in the xml.  If you want to specify the allowed
+    # fields, you can use the :fields option, which will call set_fields with
+    # the given fields:
+    #
+    #   Album.from_xml(album.to_xml, :fields=>%w'id name')
+    #
+    # If you want to update an existing instance, you can use the from_xml
+    # instance method:
+    #
+    #   album.from_xml(xml)
+    #
+    # Both of these allow creation of cached associated objects, if you provide
+    # the :associations option:
+    #
+    #   album.from_xml(xml, :associations=>:artist)
+    #
+    # You can even provide options when setting up the associated objects:
+    #
+    #   album.from_xml(xml, :associations=>{:artist=>{:fields=>%w'id name', :associations=>:tags}})
     #
     # Usage:
     #
@@ -103,32 +127,31 @@ module Sequel
 
         # Return an array of instances of this class based on
         # the provided XML.
-        def array_from_xml(xml, opts={})
-          Nokogiri::XML(xml).children.first.children.reject{|c| c.is_a?(Nokogiri::XML::Text)}.map{|c| from_xml_node(c, opts)}
+        def array_from_xml(xml, opts=OPTS)
+          node = Nokogiri::XML(xml).children.first
+          unless node 
+            raise Error, "Malformed XML used"
+          end
+          node.children.reject{|c| c.is_a?(Nokogiri::XML::Text)}.map{|c| from_xml_node(c, opts)}
         end
 
         # Return an instance of this class based on the provided
         # XML.
-        def from_xml(xml, opts={})
+        def from_xml(xml, opts=OPTS)
           from_xml_node(Nokogiri::XML(xml).children.first, opts)
         end
 
         # Return an instance of this class based on the given
         # XML node, which should be Nokogiri::XML::Node instance.
         # This should probably not be used directly by user code.
-        def from_xml_node(parent, opts={})
+        def from_xml_node(parent, opts=OPTS)
           new.from_xml_node(parent, opts)
         end
 
-        # Call the dataset +to_xml+ method.
-        def to_xml(opts={})
-          dataset.to_xml(opts)
-        end
-
         # Return an appropriate Nokogiri::XML::Builder instance
         # used to create the XML.  This should probably not be used
         # directly by user code.
-        def xml_builder(opts={})
+        def xml_builder(opts=OPTS)
           if opts[:builder]
             opts[:builder]
           else
@@ -145,7 +168,7 @@ module Sequel
         # Return a proc (or any other object that responds to []),
         # used for formatting XML tag names when serializing to XML.
         # This should probably not be used directly by user code.
-        def xml_deserialize_name_proc(opts={})
+        def xml_deserialize_name_proc(opts=OPTS)
           if opts[:name_proc]
             opts[:name_proc]
           elsif opts[:underscore]
@@ -158,7 +181,7 @@ module Sequel
         # Return a proc (or any other object that responds to []),
         # used for formatting XML tag names when serializing to XML.
         # This should probably not be used directly by user code.
-        def xml_serialize_name_proc(opts={})
+        def xml_serialize_name_proc(opts=OPTS)
           pr = if opts[:name_proc]
             opts[:name_proc]
           elsif opts[:dasherize]
@@ -170,6 +193,8 @@ module Sequel
           end
           proc{|s| "#{pr[s]}_"}
         end
+
+        Plugins.def_dataset_methods(self, :to_xml)
       end
 
       module InstanceMethods
@@ -182,36 +207,86 @@ module Sequel
         # :underscore :: Sets the :name_proc option to one that calls +underscore+
         #                on the input string.  Requires that you load the inflector
         #                extension or another library that adds String#underscore.
-        def from_xml(xml, opts={})
+        def from_xml(xml, opts=OPTS)
           from_xml_node(Nokogiri::XML(xml).children.first, opts)
         end
 
         # Update the contents of this instance based on the given 
         # XML node, which should be a Nokogiri::XML::Node instance.
-        def from_xml_node(parent, opts={})
-          cols = model.columns.map{|x| x.to_s}
-          assocs = {}
-          model.associations.map{|x| assocs[x.to_s] = model.association_reflection(x)}
-          meths = send(:setter_methods, nil, nil)
+        # By default, just calls set with a hash created from the content of the node.
+        # 
+        # Options:
+        # :associations :: Indicates that the associations cache should be updated by creating
+        #                  a new associated object using data from the hash.  Should be a Symbol
+        #                  for a single association, an array of symbols for multiple associations,
+        #                  or a hash with symbol keys and dependent association option hash values.
+        # :fields :: Changes the behavior to call set_fields using the provided fields, instead of calling set.
+        def from_xml_node(parent, opts=OPTS)
+          unless parent
+            raise Error, "Malformed XML used"
+          end
+          if !parent.children.empty? && parent.children.all?{|node| node.is_a?(Nokogiri::XML::Text)}
+            raise Error, "XML consisting of just text nodes used"
+          end
+
+          if assocs = opts[:associations]
+            assocs = case assocs
+            when Symbol
+              {assocs=>{}}
+            when Array
+              assocs_tmp = {}
+              assocs.each{|v| assocs_tmp[v] = {}}
+              assocs_tmp
+            when Hash
+              assocs
+            else
+              raise Error, ":associations should be Symbol, Array, or Hash if present"
+            end
+
+            assocs_hash = {}
+            assocs.each{|k,v| assocs_hash[k.to_s] = v}
+            assocs_present = []
+          end
+
+          hash = {}
+          populate_associations = {}
           name_proc = model.xml_deserialize_name_proc(opts)
           parent.children.each do |node|
             next if node.is_a?(Nokogiri::XML::Text)
             k = name_proc[node.name]
-            if ar = assocs[k]
-              klass = ar.associated_class
-              associations[k.to_sym] = if ar.returns_array?
-                node.children.reject{|c| c.is_a?(Nokogiri::XML::Text)}.map{|c| klass.from_xml_node(c)}
-              else
-                klass.from_xml_node(node)
-              end
-            elsif cols.include?(k)
-              self[k.to_sym] = node.key?('nil') ? nil : node.children.first.to_s
-            elsif meths.include?("#{k}=")
-              send("#{k}=", node.key?('nil') ? nil : node.children.first.to_s)
+            if assocs_hash && assocs_hash[k]
+              assocs_present << [k.to_sym, node]
             else
-              raise Error, "Entry in XML not an association or column and no setter method exists: #{k}"
+              hash[k] = node.key?('nil') ? nil : node.children.first.to_s
             end
           end
+
+          if assocs_present
+            assocs_present.each do |assoc, node|
+              assoc_opts = assocs[assoc]
+
+              unless r = model.association_reflection(assoc)
+                raise Error, "Association #{assoc} is not defined for #{model}"
+              end
+
+              populate_associations[assoc] = if r.returns_array?
+                node.children.reject{|c| c.is_a?(Nokogiri::XML::Text)}.map{|c| r.associated_class.from_xml_node(c, assoc_opts)}
+              else
+                r.associated_class.from_xml_node(node, assoc_opts)
+              end
+            end
+          end
+
+          if fields = opts[:fields]
+            set_fields(hash, fields, opts)
+          else
+            set(hash)
+          end
+
+          populate_associations.each do |assoc, values|
+            associations[assoc] = values
+          end
+
           self
         end
 
@@ -253,7 +328,7 @@ module Sequel
         #               an array of objects using Model.to_xml or Dataset#to_xml.
         # :types :: Set to true to include type information for
         #           all of the columns, pulled from the db_schema.
-        def to_xml(opts={})
+        def to_xml(opts=OPTS)
           vals = values
           types = opts[:types]
           inc = opts[:include]
@@ -292,7 +367,7 @@ module Sequel
 
         # Handle associated objects and virtual attributes when creating
         # the xml.
-        def to_xml_include(node, i, opts={})
+        def to_xml_include(node, i, opts=OPTS)
           name_proc = model.xml_serialize_name_proc(opts)
           objs = send(i)
           if objs.is_a?(Array) && objs.all?{|x| x.is_a?(Sequel::Model)}
@@ -312,12 +387,18 @@ module Sequel
         # this dataset.  Takes all of the options available to Model#to_xml,
         # as well as the :array_root_name option for specifying the name of
         # the root node that contains the nodes for all of the instances.
-        def to_xml(opts={})
+        def to_xml(opts=OPTS)
           raise(Sequel::Error, "Dataset#to_xml") unless row_proc
           x = model.xml_builder(opts)
           name_proc = model.xml_serialize_name_proc(opts)
+          array = if opts[:array]
+            opts = opts.dup
+            opts.delete(:array)
+          else
+            all
+          end
           x.send(name_proc[opts.fetch(:array_root_name, model.send(:pluralize, model.send(:underscore, model.name))).to_s]) do |x1|
-            all.each do |obj|
+            array.each do |obj|
               obj.to_xml(opts.merge(:builder=>x1))
             end
           end
