@@ -2,10 +2,13 @@ require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe "pg_array extension" do
   before(:all) do
+    Sequel.extension :pg_array
     @pg_types = Sequel::Postgres::PG_TYPES.dup
+    @pg_named_types = Sequel::Postgres::PG_NAMED_TYPES.dup
   end
   after(:all) do
     Sequel::Postgres::PG_TYPES.replace(@pg_types)
+    Sequel::Postgres::PG_NAMED_TYPES.replace(@pg_named_types)
   end
 
   before do
@@ -14,6 +17,7 @@ describe "pg_array extension" do
     @db.extension(:pg_array)
     @m = Sequel::Postgres
     @converter = @m::PG_TYPES
+    @db.sqls
   end
 
   it "should parse single dimensional text arrays" do
@@ -143,6 +147,7 @@ describe "pg_array extension" do
   end
 
   it "should literalize with types correctly" do
+    @db.literal(@m::PGArray.new([], :int4)).should == "'{}'::int4[]"
     @db.literal(@m::PGArray.new([1], :int4)).should == 'ARRAY[1]::int4[]'
     @db.literal(@m::PGArray.new([nil], :text)).should == 'ARRAY[NULL]::text[]'
     @db.literal(@m::PGArray.new([nil, 1], :int8)).should == 'ARRAY[NULL,1]::int8[]'
@@ -226,7 +231,7 @@ describe "pg_array extension" do
       end
 
       @db.literal(@db.typecast_value(meth, [array_in])).should == "ARRAY[#{output}]::#{db_type}[]"
-      @db.literal(@db.typecast_value(meth, [])).should == "ARRAY[]::#{db_type}[]"
+      @db.literal(@db.typecast_value(meth, [])).should == "'{}'::#{db_type}[]"
     end
     proc{@db.typecast_value(:integer_array, {})}.should raise_error(Sequel::InvalidValue)
   end
@@ -300,14 +305,14 @@ describe "pg_array extension" do
     Sequel::Postgres::PG_TYPES[2].should be_a_kind_of(Sequel::Postgres::PGArray::JSONCreator)
   end
 
-  it "should support registering converters with :parser=>:json option" do
+  it "should support registering converters with :parser=>:json option and blocks" do
     Sequel::Postgres::PGArray.register('foo', :oid=>4, :parser=>:json){|s| s * 2}
     Sequel::Postgres::PG_TYPES[4].call('{{1, 2}, {3, 4}}').should == [[2, 4], [6, 8]]
   end
 
   it "should support registering custom types with :array_type option" do
     Sequel::Postgres::PGArray.register('foo', :oid=>3, :array_type=>:blah)
-    @db.literal(Sequel::Postgres::PG_TYPES[3].call('{}')).should == 'ARRAY[]::blah[]'
+    @db.literal(Sequel::Postgres::PG_TYPES[3].call('{}')).should == "'{}'::blah[]"
   end
 
   it "should use and not override existing database typecast method if :typecast_method option is given" do
@@ -316,9 +321,60 @@ describe "pg_array extension" do
     @db.schema(:items).map{|e| e[1][:type]}.should == [:float_array]
   end
 
-  it "should set appropriate timestamp conversion procs when getting conversion procs" do
+  it "should support registering custom array types on a per-Database basis" do
+    @db.register_array_type('banana', :oid=>7865){|s| s}
+    @db.typecast_value(:banana_array, []).should be_a_kind_of(Sequel::Postgres::PGArray)
+    @db.fetch = [{:name=>'id', :db_type=>'banana[]'}]
+    @db.schema(:items).map{|e| e[1][:type]}.should == [:banana_array]
+    @db.conversion_procs.should have_key(7865)
+    @db.respond_to?(:typecast_value_banana_array, true).should == true
+
+    db = Sequel.connect('mock://postgres', :quote_identifiers=>false)
+    db.extend_datasets(Module.new{def supports_timestamp_timezones?; false; end; def supports_timestamp_usecs?; false; end})
+    db.extension(:pg_array)
+    db.fetch = [{:name=>'id', :db_type=>'banana[]'}]
+    db.schema(:items).map{|e| e[1][:type]}.should == [nil]
+    db.conversion_procs.should_not have_key(7865)
+    db.respond_to?(:typecast_value_banana_array, true).should == false
+  end
+
+  it "should automatically look up the array and scalar oids when registering per-Database types" do
+    @db.fetch = [[{:oid=>21, :typarray=>7866}], [{:name=>'id', :db_type=>'banana[]'}]]
+    @db.register_array_type('banana', :scalar_typecast=>:integer)
+    @db.sqls.should == ["SELECT typarray, oid FROM pg_type WHERE (typname = 'banana') LIMIT 1"]
+    @db.schema(:items).map{|e| e[1][:type]}.should == [:banana_array]
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should not automatically look up oids if given both scalar and array oids" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_oid=>21, :scalar_typecast=>:integer)
+    @db.sqls.should == []
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should not automatically look up oids if given array oid and block" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_typecast=>:integer){|s| s.to_i}
+    @db.sqls.should == []
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should set appropriate timestamp conversion procs when resetting conversion procs" do
+    Sequel::Postgres::PG_NAMED_TYPES[:foo] = proc{|v| v*2}
+    @db.fetch = [[{:oid=>2222, :typname=>'foo'}], [{:oid=>2222, :typarray=>2223, :typname=>'foo'}]]
+    @db.reset_conversion_procs
     procs = @db.conversion_procs
     procs[1185].call('{"2011-10-20 11:12:13"}').should == [Time.local(2011, 10, 20, 11, 12, 13)]
     procs[1115].call('{"2011-10-20 11:12:13"}').should == [Time.local(2011, 10, 20, 11, 12, 13)]
+    procs[2222].call('1').should == '11'
+    procs[2223].call('{"2"}').should == ['22']
+  end
+
+  it "should return correct results for Database#schema_type_class" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_typecast=>:integer){|s| s.to_i}
+    @db.schema_type_class(:banana_array).should == Sequel::Postgres::PGArray
+    @db.schema_type_class(:integer).should == Integer
   end
 end
