@@ -2,7 +2,7 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 describe "Simple Dataset operations" do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:items) do
       primary_key :id
       Integer :number
@@ -50,7 +50,7 @@ describe "Simple Dataset operations" do
   end
 
   specify "should have insert_multiple return primary key values" do
-    @ds.insert_multiple([{:number=>20}, {:number=>30}]).should == [2, 3]
+    @ds.extension(:sequel_3_dataset_methods).insert_multiple([{:number=>20}, {:number=>30}]).should == [2, 3]
     @ds.filter(:id=>2).get(:number).should == 20
     @ds.filter(:id=>3).get(:number).should == 30
   end
@@ -59,17 +59,27 @@ describe "Simple Dataset operations" do
     @ds.join(:items___b, :id=>:id).select_all(:items).all.should == [{:id=>1, :number=>10}]
   end
 
+  specify "should handle LATERAL subqueries correctly" do
+    @ds << {:number=>20}
+    @ds.from(:items___i, @ds.where(:items__number=>:i__number).lateral).select_order_map([:i__number___n, :t1__number]).should == [[10, 10], [20, 20]]
+    @ds.from(:items___i).cross_join(@ds.where(:items__number=>:i__number).lateral).select_order_map([:i__number___n, :t1__number]).should == [[10, 10], [20, 20]]
+    @ds.from(:items___i).join(@ds.where(:items__number=>:i__number).lateral, 1=>1).select_order_map([:i__number___n, :t1__number]).should == [[10, 10], [20, 20]]
+    @ds.from(:items___i).join(@ds.where(:items__number=>:i__number).lateral, 1=>0).select_order_map([:i__number___n, :t1__number]).should == []
+    @ds.from(:items___i).left_join(@ds.from(:items___i2).where(:i2__number=>:i__number).lateral, 1=>1).select_order_map([:i__number___n, :t1__number]).should == [[10, 10], [20, 20]]
+    @ds.from(:items___i).left_join(@ds.from(:items___i2).where(:i2__number=>:i__number).lateral, 1=>0).select_order_map([:i__number___n, :t1__number]).should == [[10, nil], [20, nil]]
+  end if DB.dataset.supports_lateral_subqueries?
+
   specify "should correctly deal with qualified columns and subselects" do
     @ds.from_self(:alias=>:a).select(:a__id, Sequel.qualify(:a, :number)).all.should == [{:id=>1, :number=>10}]
     @ds.join(@ds.as(:a), :id=>:id).select(:a__id, Sequel.qualify(:a, :number)).all.should == [{:id=>1, :number=>10}]
   end
 
   specify "should graph correctly" do
-    @ds.graph(:items, {:id=>:id}, :table_alias=>:b).all.should == [{:items=>{:id=>1, :number=>10}, :b=>{:id=>1, :number=>10}}]
+    @ds.graph(:items, {:id=>:id}, :table_alias=>:b).extension(:graph_each).all.should == [{:items=>{:id=>1, :number=>10}, :b=>{:id=>1, :number=>10}}]
   end
 
   specify "should graph correctly with a subselect" do
-    @ds.from_self(:alias=>:items).graph(@ds.from_self, {:id=>:id}, :table_alias=>:b).all.should == [{:items=>{:id=>1, :number=>10}, :b=>{:id=>1, :number=>10}}]
+    @ds.from_self(:alias=>:items).graph(@ds.from_self, {:id=>:id}, :table_alias=>:b).extension(:graph_each).all.should == [{:items=>{:id=>1, :number=>10}, :b=>{:id=>1, :number=>10}}]
   end
 
   cspecify "should have insert work correctly when inserting a row with all NULL values", :hsqldb do
@@ -98,6 +108,28 @@ describe "Simple Dataset operations" do
     @ds.all.should == [{:id=>1, :number=>10}]
   end
 
+  specify "should iterate over records as they come in" do
+    called = false
+    @ds.each{|row| called = true; row.should == {:id=>1, :number=>10}}
+    called.should == true
+  end
+
+  specify "should support iterating over large numbers of records with paged_each" do
+    (2..100).each{|i| @ds.insert(:number=>i*10)}
+
+    rows = []
+    @ds.order(:number).paged_each(:rows_per_fetch=>5){|row| rows << row}
+    rows.should == (1..100).map{|i| {:id=>i, :number=>i*10}}
+
+    rows = []
+    @ds.order(:number).paged_each(:rows_per_fetch=>3){|row| rows << row}
+    rows.should == (1..100).map{|i| {:id=>i, :number=>i*10}}
+
+    rows = []
+    @ds.order(:number).limit(50, 25).paged_each(:rows_per_fetch=>3){|row| rows << row}
+    rows.should == (26..75).map{|i| {:id=>i, :number=>i*10}}
+  end
+
   specify "should fetch all results correctly" do
     @ds.all.should == [{:id=>1, :number=>10}]
   end
@@ -125,6 +157,15 @@ describe "Simple Dataset operations" do
     @ds.order(:id).limit(2, 0).all.should == [{:id=>1, :number=>10}, {:id=>2, :number=>20}]
     @ds.order(:id).limit(2, 1).all.should == [{:id=>2, :number=>20}]
   end
+
+  specify "should fetch correctly with a limit and offset using seperate methods" do
+    @ds.order(:id).limit(2).offset(0).all.should == [{:id=>1, :number=>10}]
+    @ds.order(:id).limit(2).offset(1).all.should == []
+    @ds.insert(:number=>20)
+    @ds.order(:id).limit(1).offset(1).all.should == [{:id=>2, :number=>20}]
+    @ds.order(:id).limit(2).offset(0).all.should == [{:id=>1, :number=>10}, {:id=>2, :number=>20}]
+    @ds.order(:id).limit(2).offset(1).all.should == [{:id=>2, :number=>20}]
+  end
   
   specify "should provide correct columns when using a limit and offset" do
     ds = @ds.order(:id).limit(1, 1)
@@ -154,6 +195,12 @@ describe "Simple Dataset operations" do
     @ds.limit(2, 1).all.should == []
   end
 
+  specify "should be orderable by column number" do
+    @ds.insert(:number=>20)
+    @ds.insert(:number=>10)
+    @ds.order(2, 1).select_map([:id, :number]).should == [[1, 10], [3, 10], [2, 20]]
+  end
+
   specify "should fetch correctly with a limit in an IN subselect" do
     @ds.where(:id=>@ds.select(:id).order(:id).limit(2)).all.should == [{:id=>1, :number=>10}]
     @ds.insert(:number=>20)
@@ -170,6 +217,32 @@ describe "Simple Dataset operations" do
     @ds.where(:id=>@ds.select(:id).order(:id).limit(2, 1)).all.should == [{:id=>2, :number=>20}]
   end
   
+  specify "should fetch correctly when using limit and offset in a from_self" do
+    @ds.insert(:number=>20)
+    ds = @ds.order(:id).limit(1, 1).from_self
+    ds.all.should == [{:number=>20, :id=>2}]
+    ds.columns.should == [:id, :number]
+    @ds.order(:id).limit(1, 1).columns.should == [:id, :number]
+  end
+
+  specify "should fetch correctly when using nested limit and offset in a from_self" do
+    @ds.insert(:number=>20)
+    @ds.insert(:number=>30)
+    ds = @ds.order(:id).limit(2, 1).from_self.reverse_order(:number).limit(1, 1)
+    ds.all.should == [{:number=>20, :id=>2}]
+    ds.columns.should == [:id, :number]
+    @ds.order(:id).limit(2, 1).from_self.reverse_order(:number).limit(1, 1).columns.should == [:id, :number]
+
+    ds = @ds.order(:id).limit(3, 1).from_self.limit(2, 1).from_self.limit(1, 1)
+    ds.all.should == []
+    ds.columns.should == [:id, :number]
+
+    @ds.insert(:number=>40)
+    ds = @ds.order(:id).limit(3, 1).from_self.reverse_order(:number).limit(2, 1).from_self.reverse_order(:id).limit(1, 1)
+    ds.all.should == [{:number=>20, :id=>2}]
+    ds.columns.should == [:id, :number]
+  end
+
   specify "should alias columns correctly" do
     @ds.select(:id___x, :number___n).first.should == {:x=>1, :n=>10}
   end
@@ -184,7 +257,7 @@ end
 
 describe "Simple dataset operations with nasty table names" do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @table = :"i`t' [e]\"m\\s" 
     @qi = @db.quote_identifiers?
     @db.quote_identifiers = true
@@ -193,7 +266,7 @@ describe "Simple dataset operations with nasty table names" do
     @db.quote_identifiers = @qi
   end
 
-  cspecify "should work correctly", :mssql, :oracle do
+  cspecify "should work correctly", :mssql, :oracle, :sqlanywhere do
     @db.create_table!(@table) do
       primary_key :id
       Integer :number
@@ -211,14 +284,14 @@ end
 
 describe Sequel::Dataset do
   before do
-    INTEGRATION_DB.create_table!(:test) do
+    DB.create_table!(:test) do
       String :name
       Integer :value
     end
-    @d = INTEGRATION_DB[:test]
+    @d = DB[:test]
   end
   after do
-    INTEGRATION_DB.drop_table?(:test)
+    DB.drop_table?(:test)
   end
 
   specify "should return the correct record count" do
@@ -301,44 +374,53 @@ describe Sequel::Database do
      "\\'dingo",
      "\\\\''dingo",
     ].each do |str|
-      INTEGRATION_DB.get(Sequel.cast(str, String)).should == str
+      DB.get(Sequel.cast(str, String)).should == str
       str = "1#{str}1"
-      INTEGRATION_DB.get(Sequel.cast(str, String)).should == str
+      DB.get(Sequel.cast(str, String)).should == str
       str = "#{str}#{str}"
-      INTEGRATION_DB.get(Sequel.cast(str, String)).should == str
+      DB.get(Sequel.cast(str, String)).should == str
     end
   end
 
   cspecify "should properly escape binary data", [:odbc], [:jdbc, :hsqldb], :oracle do
-    INTEGRATION_DB.get(Sequel.cast(Sequel.blob("\1\2\3"), File).as(:a)).should == "\1\2\3"
+    DB.get(Sequel.cast(Sequel.blob("\1\2\3"), File).as(:a)).should == "\1\2\3"
+  end
+
+  cspecify "should properly handle empty blobs", [:jdbc, :hsqldb], :oracle do
+    DB.get(Sequel.cast(Sequel.blob(""), File).as(:a)).should == ""
+  end
+
+  cspecify "should properly escape identifiers", :db2, :oracle, :sqlanywhere do
+    DB.create_table(:"\\'\"[]"){Integer :id}
+    DB.drop_table(:"\\'\"[]")
   end
 
   specify "should have a working table_exists?" do
     t = :basdfdsafsaddsaf
-    INTEGRATION_DB.drop_table?(t)
-    INTEGRATION_DB.table_exists?(t).should == false
-    INTEGRATION_DB.create_table(t){Integer :a}
+    DB.drop_table?(t)
+    DB.table_exists?(t).should == false
+    DB.create_table(t){Integer :a}
     begin
-      INTEGRATION_DB.table_exists?(t).should == true
+      DB.table_exists?(t).should == true
     ensure
-      INTEGRATION_DB.drop_table(t)
+      DB.drop_table(t)
     end
   end
 end
 
 describe Sequel::Dataset do
   before do
-    INTEGRATION_DB.create_table! :items do
+    DB.create_table! :items do
       primary_key :id 
       Integer :value
     end 
-    @d = INTEGRATION_DB[:items]
+    @d = DB[:items]
     @d << {:value => 123}
     @d << {:value => 456}
     @d << {:value => 789}
   end 
   after do
-    INTEGRATION_DB.drop_table?(:items)
+    DB.drop_table?(:items)
   end 
   
   specify "should correctly return avg" do
@@ -360,14 +442,14 @@ end
 
 describe "Simple Dataset operations" do
   before do
-    INTEGRATION_DB.create_table!(:items) do
+    DB.create_table!(:items) do
       Integer :number
       TrueClass :flag
     end
-    @ds = INTEGRATION_DB[:items]
+    @ds = DB[:items]
   end
   after do
-    INTEGRATION_DB.drop_table?(:items)
+    DB.drop_table?(:items)
   end
 
   specify "should deal with boolean conditions correctly" do
@@ -386,18 +468,18 @@ end
 
 describe "Simple Dataset operations in transactions" do
   before do
-    INTEGRATION_DB.create_table!(:items) do
+    DB.create_table!(:items) do
       primary_key :id
       integer :number
     end
-    @ds = INTEGRATION_DB[:items]
+    @ds = DB[:items]
   end
   after do
-    INTEGRATION_DB.drop_table?(:items)
+    DB.drop_table?(:items)
   end
 
   cspecify "should insert correctly with a primary key specified inside a transaction", :db2, :mssql do
-    INTEGRATION_DB.transaction do
+    DB.transaction do
       @ds.insert(:id=>100, :number=>20)
       @ds.count.should == 1
       @ds.order(:id).all.should == [{:id=>100, :number=>20}]
@@ -405,7 +487,7 @@ describe "Simple Dataset operations in transactions" do
   end
   
   specify "should have insert return primary key value inside a transaction" do
-    INTEGRATION_DB.transaction do
+    DB.transaction do
       @ds.insert(:number=>20).should == 1
       @ds.count.should == 1
       @ds.order(:id).all.should == [{:id=>1, :number=>20}]
@@ -413,23 +495,23 @@ describe "Simple Dataset operations in transactions" do
   end
   
   specify "should support for_update" do
-    INTEGRATION_DB.transaction{@ds.for_update.all.should == []}
+    DB.transaction{@ds.for_update.all.should == []}
   end
 end
 
 describe "Dataset UNION, EXCEPT, and INTERSECT" do
   before do
-    INTEGRATION_DB.create_table!(:i1){integer :number}
-    INTEGRATION_DB.create_table!(:i2){integer :number}
-    @ds1 = INTEGRATION_DB[:i1]
+    DB.create_table!(:i1){integer :number}
+    DB.create_table!(:i2){integer :number}
+    @ds1 = DB[:i1]
     @ds1.insert(:number=>10)
     @ds1.insert(:number=>20)
-    @ds2 = INTEGRATION_DB[:i2]
+    @ds2 = DB[:i2]
     @ds2.insert(:number=>10)
     @ds2.insert(:number=>30)
   end
   after do
-    INTEGRATION_DB.drop_table?(:i1, :i2, :i3)
+    DB.drop_table?(:i1, :i2, :i3)
   end
   
   specify "should give the correct results for simple UNION, EXCEPT, and INTERSECT" do
@@ -463,8 +545,8 @@ describe "Dataset UNION, EXCEPT, and INTERSECT" do
   end
 
   specify "should give the correct results for compound UNION, EXCEPT, and INTERSECT" do
-    INTEGRATION_DB.create_table!(:i3){integer :number}
-    @ds3 = INTEGRATION_DB[:i3]
+    DB.create_table!(:i3){integer :number}
+    @ds3 = DB[:i3]
     @ds3.insert(:number=>10)
     @ds3.insert(:number=>40)
 
@@ -493,10 +575,10 @@ describe "Dataset UNION, EXCEPT, and INTERSECT" do
   end
 end
 
-if INTEGRATION_DB.dataset.supports_cte?
+if DB.dataset.supports_cte?
   describe "Common Table Expressions" do
     before(:all) do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:i1){Integer :id; Integer :parent_id}
       @ds = @db[:i1]
       @ds.insert(:id=>1)
@@ -515,9 +597,9 @@ if INTEGRATION_DB.dataset.supports_cte?
     end
     
     cspecify "should give correct results for recursive WITH", :db2 do
-      ds = @db[:t].select(:i___id, :pi___parent_id).with_recursive(:t, @ds.filter(:parent_id=>nil), @ds.join(:t, :i=>:parent_id).select(:i1__id, :i1__parent_id), :args=>[:i, :pi])
+      ds = @db[:t].select(:i___id, :pi___parent_id).with_recursive(:t, @ds.filter(:parent_id=>nil), @ds.join(:t, :i=>:parent_id).select(:i1__id, :i1__parent_id), :args=>[:i, :pi]).order(:id)
       ds.all.should == [{:parent_id=>nil, :id=>1}, {:parent_id=>nil, :id=>2}, {:parent_id=>1, :id=>3}, {:parent_id=>1, :id=>4}, {:parent_id=>3, :id=>5}, {:parent_id=>5, :id=>6}]
-      ps = @db[:t].select(:i___id, :pi___parent_id).with_recursive(:t, @ds.filter(:parent_id=>:$n), @ds.join(:t, :i=>:parent_id).filter(:t__i=>:parent_id).select(:i1__id, :i1__parent_id), :args=>[:i, :pi]).prepare(:select, :cte_sel)
+      ps = @db[:t].select(:i___id, :pi___parent_id).with_recursive(:t, @ds.filter(:parent_id=>:$n), @ds.join(:t, :i=>:parent_id).filter(:t__i=>:parent_id).select(:i1__id, :i1__parent_id), :args=>[:i, :pi]).order(:id).prepare(:select, :cte_sel)
       ps.call(:n=>1).should == [{:id=>3, :parent_id=>1}, {:id=>4, :parent_id=>1}, {:id=>5, :parent_id=>3}, {:id=>6, :parent_id=>5}]
       ps.call(:n=>3).should == [{:id=>5, :parent_id=>3}, {:id=>6, :parent_id=>5}]
       ps.call(:n=>5).should == [{:id=>6, :parent_id=>5}]
@@ -548,10 +630,10 @@ if INTEGRATION_DB.dataset.supports_cte?
   end
 end
 
-if INTEGRATION_DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
+if DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
   describe "Common Table Expressions in INSERT/UPDATE/DELETE" do
     before do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:i1){Integer :id}
       @ds = @db[:i1]
       @ds2 = @ds.with(:t, @ds)
@@ -573,10 +655,10 @@ if INTEGRATION_DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE supp
   end
 end
 
-if INTEGRATION_DB.dataset.supports_returning?(:insert)
+if DB.dataset.supports_returning?(:insert)
   describe "RETURNING clauses in INSERT" do
     before do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:i1){Integer :id; Integer :foo}
       @ds = @db[:i1]
     end
@@ -598,10 +680,10 @@ if INTEGRATION_DB.dataset.supports_returning?(:insert)
   end
 end
 
-if INTEGRATION_DB.dataset.supports_returning?(:update) # Assume DELETE support as well
+if DB.dataset.supports_returning?(:update) # Assume DELETE support as well
   describe "RETURNING clauses in UPDATE/DELETE" do
     before do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:i1){Integer :id; Integer :foo}
       @ds = @db[:i1]
       @ds.insert(1, 2)
@@ -634,10 +716,10 @@ if INTEGRATION_DB.dataset.supports_returning?(:update) # Assume DELETE support a
   end
 end
 
-if INTEGRATION_DB.dataset.supports_window_functions?
+if DB.dataset.supports_window_functions?
   describe "Window Functions" do
     before(:all) do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:i1){Integer :id; Integer :group_id; Integer :amount}
       @ds = @db[:i1].order(:id)
       @ds.insert(:id=>1, :group_id=>1, :amount=>1)
@@ -688,7 +770,7 @@ end
 
 describe Sequel::SQL::Constants do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @ds = @db[:constants]
     @c = proc do |v|
       case v
@@ -735,7 +817,7 @@ end
 
 describe "Sequel::Dataset#import and #multi_insert" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:imp){Integer :i}
     @ids = @db[:imp].order(:i)
   end
@@ -778,7 +860,7 @@ end
 
 describe "Sequel::Dataset#import and #multi_insert :return=>:primary_key " do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:imp){primary_key :id; Integer :i}
     @ds = @db[:imp]
   end
@@ -801,7 +883,7 @@ end
 
 describe "Sequel::Dataset convenience methods" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :a; Integer :b; Integer :c}
     @ds = @db[:a]
     @ds.insert(1, 3, 5)
@@ -817,17 +899,17 @@ describe "Sequel::Dataset convenience methods" do
   it "#group_rollup should include hierarchy of groupings" do
     @ds.group_by(:a).group_rollup.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.should == [[1, 10, 16], [2, 7, 11], [nil, 17, 27]]
     @ds.group_by(:a, :b).group_rollup.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.should == [[1, 3, 11], [1, 4, 5], [1, nil, 16], [2, 3, 5], [2, 4, 6], [2, nil, 11], [nil, nil, 27]]
-  end if INTEGRATION_DB.dataset.supports_group_rollup?
+  end if DB.dataset.supports_group_rollup?
 
   it "#group_cube should include all combinations of groupings" do
     @ds.group_by(:a).group_cube.select_map([:a, Sequel.function(:sum, :b).cast(Integer).as(:b), Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.should == [[1, 10, 16], [2, 7, 11], [nil, 17, 27]]
     @ds.group_by(:a, :b).group_cube.select_map([:a, :b, Sequel.function(:sum, :c).cast(Integer).as(:c)]).sort_by{|x| x.inspect}.should == [[1, 3, 11], [1, 4, 5], [1, nil, 16], [2, 3, 5], [2, 4, 6], [2, nil, 11], [nil, 3, 16], [nil, 4, 11], [nil, nil, 27]]
-  end if INTEGRATION_DB.dataset.supports_group_cube?
+  end if DB.dataset.supports_group_cube?
 end
 
 describe "Sequel::Dataset convenience methods" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :a; Integer :b}
     @ds = @db[:a].order(:a)
   end
@@ -840,7 +922,7 @@ describe "Sequel::Dataset convenience methods" do
   
   it "#[]= should update matching rows" do
     @ds.insert(20, 10)
-    @ds[:a=>20] = {:b=>30}
+    @ds.extension(:sequel_3_dataset_methods)[:a=>20] = {:b=>30}
     @ds.all.should == [{:a=>20, :b=>30}]
   end
   
@@ -905,7 +987,7 @@ end
   
 describe "Sequel::Dataset main SQL methods" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:d){Integer :a; Integer :b}
     @ds = @db[:d].order(:a)
   end
@@ -976,7 +1058,7 @@ end
 
 describe "Sequel::Dataset convenience methods" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :a; Integer :b; Integer :c; Integer :d}
     @ds = @db[:a].order(:a)
   end
@@ -1078,7 +1160,7 @@ end
 
 describe "Sequel::Dataset DSL support" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :a; Integer :b}
     @ds = @db[:a].order(:a)
   end
@@ -1272,7 +1354,7 @@ describe "Sequel::Dataset DSL support" do
     @ds.filter([:a, :b]=>[]).all.should == []
     @ds.exclude([:a, :b]=>[]).all.should == []
 
-    unless Sequel.guarded?(:mssql, :oracle, :db2)
+    unless Sequel.guarded?(:mssql, :oracle, :db2, :sqlanywhere)
       # Some databases don't like boolean results in the select list
       pr = proc{|r| r.is_a?(Integer) ? (r != 0) : r}
       pr[@ds.get(Sequel.expr(:a=>[]))].should == nil
@@ -1282,25 +1364,21 @@ describe "Sequel::Dataset DSL support" do
     end
   end
   
-  specify "should work empty arrays with nulls and Sequel.empty_array_null_handling = true" do
-    begin
-      Sequel.empty_array_handle_nulls = false
-      @ds.insert(nil, nil)
-      @ds.filter(:a=>[]).all.should == []
-      @ds.exclude(:a=>[]).all.should == [{:a=>nil, :b=>nil}]
-      @ds.filter([:a, :b]=>[]).all.should == []
-      @ds.exclude([:a, :b]=>[]).all.should == [{:a=>nil, :b=>nil}]
+  specify "should work empty arrays with nulls and the empty_array_ignore_nulls extension" do
+    ds = @ds.extension(:empty_array_ignore_nulls)
+    ds.insert(nil, nil)
+    ds.filter(:a=>[]).all.should == []
+    ds.exclude(:a=>[]).all.should == [{:a=>nil, :b=>nil}]
+    ds.filter([:a, :b]=>[]).all.should == []
+    ds.exclude([:a, :b]=>[]).all.should == [{:a=>nil, :b=>nil}]
 
-      unless Sequel.guarded?(:mssql, :oracle, :db2)
-        # Some databases don't like boolean results in the select list
-        pr = proc{|r| r.is_a?(Integer) ? (r != 0) : r}
-        pr[@ds.get(Sequel.expr(:a=>[]))].should == false
-        pr[@ds.get(~Sequel.expr(:a=>[]))].should == true
-        pr[@ds.get(Sequel.expr([:a, :b]=>[]))].should == false
-        pr[@ds.get(~Sequel.expr([:a, :b]=>[]))].should == true
-      end
-    ensure
-      Sequel.empty_array_handle_nulls = true
+    unless Sequel.guarded?(:mssql, :oracle, :db2, :sqlanywhere)
+      # Some databases don't like boolean results in the select list
+      pr = proc{|r| r.is_a?(Integer) ? (r != 0) : r}
+      pr[ds.get(Sequel.expr(:a=>[]))].should == false
+      pr[ds.get(~Sequel.expr(:a=>[]))].should == true
+      pr[ds.get(Sequel.expr([:a, :b]=>[]))].should == false
+      pr[ds.get(~Sequel.expr([:a, :b]=>[]))].should == true
     end
   end
 
@@ -1316,7 +1394,7 @@ end
 
 describe "SQL Extract Function" do
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){DateTime :a}
     @ds = @db[:a].order(:a)
   end
@@ -1339,7 +1417,7 @@ end
 
 describe "Dataset string methods" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     csc = {}
     cic = {}
     csc[:collate] = @db.dataset_class::CASE_SENSITIVE_COLLATION if defined? @db.dataset_class::CASE_SENSITIVE_COLLATION
@@ -1416,7 +1494,35 @@ describe "Dataset string methods" do
     @ds.exclude(Sequel.expr(:a).ilike('FOO', 'BAR')).all.should == []
   end
   
-  if INTEGRATION_DB.dataset.supports_regexp?
+  it "#escape_like should escape any metacharacters" do
+    @ds.insert('foo', 'bar')
+    @ds.insert('foo.', 'bar..')
+    @ds.insert('foo\\..', 'bar\\..')
+    @ds.insert('foo\\_', 'bar\\%')
+    @ds.insert('foo_', 'bar%')
+    @ds.insert('foo_.', 'bar%.')
+    @ds.insert('foo_..', 'bar%..')
+    @ds.insert('[f#*?oo_]', '[bar%]')
+    @ds.filter(Sequel.expr(:a).like(@ds.escape_like('foo_'))).select_order_map(:a).should == ['foo_']
+    @ds.filter(Sequel.expr(:b).like(@ds.escape_like('bar%'))).select_order_map(:b).should == ['bar%']
+    @ds.filter(Sequel.expr(:a).like(@ds.escape_like('foo\\_'))).select_order_map(:a).should == ['foo\\_']
+    @ds.filter(Sequel.expr(:b).like(@ds.escape_like('bar\\%'))).select_order_map(:b).should == ['bar\\%']
+    @ds.filter(Sequel.expr(:a).like(@ds.escape_like('[f#*?oo_]'))).select_order_map(:a).should == ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).like(@ds.escape_like('[bar%]'))).select_order_map(:b).should == ['[bar%]']
+    @ds.filter(Sequel.expr(:b).like("#{@ds.escape_like('bar%')}_")).select_order_map(:b).should == ['bar%.']
+    @ds.filter(Sequel.expr(:b).like("#{@ds.escape_like('bar%')}%")).select_order_map(:b).should == ['bar%', 'bar%.', 'bar%..']
+
+    @ds.filter(Sequel.expr(:a).ilike(@ds.escape_like('Foo_'))).select_order_map(:a).should == ['foo_']
+    @ds.filter(Sequel.expr(:b).ilike(@ds.escape_like('Bar%'))).select_order_map(:b).should == ['bar%']
+    @ds.filter(Sequel.expr(:a).ilike(@ds.escape_like('Foo\\_'))).select_order_map(:a).should == ['foo\\_']
+    @ds.filter(Sequel.expr(:b).ilike(@ds.escape_like('Bar\\%'))).select_order_map(:b).should == ['bar\\%']
+    @ds.filter(Sequel.expr(:a).ilike(@ds.escape_like('[F#*?oo_]'))).select_order_map(:a).should == ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).ilike(@ds.escape_like('[Bar%]'))).select_order_map(:b).should == ['[bar%]']
+    @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}_")).select_order_map(:b).should == ['bar%.']
+    @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}%")).select_order_map(:b).should == ['bar%', 'bar%.', 'bar%..']
+  end
+  
+  if DB.dataset.supports_regexp?
     it "#like with regexp return matching rows" do
       @ds.insert('foo', 'bar')
       @ds.filter(Sequel.expr(:a).like(/fo/)).all.should == [{:a=>'foo', :b=>'bar'}]
@@ -1469,7 +1575,7 @@ describe "Dataset identifier methods" do
         upcase.reverse
       end
     end
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :ab}
     @db[:a].insert(1)
   end
@@ -1500,9 +1606,9 @@ end
 
 describe "Dataset defaults and overrides" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){Integer :a}
-    @ds = @db[:a].order(:a)
+    @ds = @db[:a].order(:a).extension(:set_overrides)
   end
   before do
     @ds.delete
@@ -1526,14 +1632,14 @@ describe "Dataset defaults and overrides" do
   end
 end
 
-if INTEGRATION_DB.dataset.supports_modifying_joins?
+if DB.dataset.supports_modifying_joins?
   describe "Modifying joined datasets" do
     before do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:a){Integer :a; Integer :d}
       @db.create_table!(:b){Integer :b; Integer :e}
       @db.create_table!(:c){Integer :c; Integer :f}
-      @ds = @db.from(:a, :b).join(:c, :c=>Sequel.identifier(:e)).where(:d=>:b, :f=>6)
+      @ds = @db.from(:a, :b).join(:c, {:c=>Sequel.identifier(:e)}, :qualify=>:symbol).where(:d=>:b, :f=>6)
       @db[:a].insert(1, 2)
       @db[:a].insert(3, 4)
       @db[:b].insert(2, 5)
@@ -1565,7 +1671,7 @@ end
 
 describe "Emulated functions" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:a){String :a}
     @ds = @db[:a]
   end
@@ -1594,3 +1700,39 @@ describe "Emulated functions" do
     @ds.get(Sequel.trim(:a)).should == 'foo22'
   end
 end
+
+describe "Dataset replace" do
+  before do
+    DB.create_table(:items){Integer :id, :unique=>true; Integer :value}
+    sqls = []
+    DB.loggers << Class.new{%w'info error'.each{|m| define_method(m){|sql| sqls << sql}}}.new
+
+    @d = DB[:items]
+    sqls.clear
+  end
+
+  after do
+    DB.drop_table?(:items)
+  end
+
+  specify "should use support arrays, datasets, and multiple values" do
+    @d.replace([1, 2])
+    @d.all.should == [{:id=>1, :value=>2}]
+    @d.replace(1, 2)
+    @d.all.should == [{:id=>1, :value=>2}]
+    @d.replace(@d)
+    @d.all.should == [{:id=>1, :value=>2}]
+  end
+
+  specify "should create a record if the condition is not met" do
+    @d.replace(:id => 111, :value => 333)
+    @d.all.should == [{:id => 111, :value => 333}]
+  end
+
+  specify "should update a record if the condition is met" do
+    @d << {:id => 111}
+    @d.all.should == [{:id => 111, :value => nil}]
+    @d.replace(:id => 111, :value => 333)
+    @d.all.should == [{:id => 111, :value => 333}]
+  end
+end if DB.dataset.supports_replace?
